import createLeafletObject from './leaflet-geometry';

const leafletObjectMap = new Map();

/**
 * Creates a new leaflet object and adds it to the leaflet object map with the given row ID.
 * @param {HTMLElement} node - The HTML element containing the row ID in its dataset.
 * @returns {Array} An array containing the newly created leaflet object.
 */
function addNodeToHyperleaflet(node) {
  const { dataset } = node;
  const rowId = dataset.id;

  if (rowId in leafletObjectMap) {
    // eslint-disable-next-line no-console
    console.error(`%c${rowId}`, 'color:red', 'already exists', node);
    return [];
  }

  const leafletObject = createLeafletObject({ ...dataset });
  leafletObjectMap.set(rowId, leafletObject);

  return [leafletObject];
}
/**
 * Removes the leaflet object associated with the given HTML element from the leaflet object map.
 * @param {HTMLElement} node - The HTML element containing the row ID in its dataset.
 * @returns {Array} An array containing the removed leaflet object.
 */
function deleteNodeFromHyperleaflet(node) {
  const rowId = node.dataset.id;
  const leafletObject = leafletObjectMap.get(rowId);
  leafletObjectMap.delete(rowId);
  return [leafletObject];
}

/**
 * Initializes the hyperleaflet geometry handler with the given map and callbacks.
 * @param {Object} map - The leaflet map instance to be used.
 * @param {Object} callbacks - An object containing the add and remove callbacks to be called when elements are added or removed from the map.
 * @param {Function} callbacks.addCallback - The function to be called when an element is added to the map.
 * @param {Function} callbacks.removeCallback - The function to be called when an element is removed from the map.
 * @returns {Object} An object containing the addNoteListToHyperleaflet and removeNodeListToHyperleaflet functions.
 */
export default function hyperleafletGeometryHandler(map, { addCallback = () => {}, removeCallback = () => {} }) {
  /**
   * Adds a list of nodes to the map as Leaflet objects.
   * @param {Array} nodes - An array of nodes to be added to the map.
   */
  const addNoteListToHyperleaflet = (nodes) => {
    nodes.forEach((node) => {
      if (node.nodeType === 1 && node.matches('[data-id]')) {
        const [leafletObject] = addNodeToHyperleaflet(node);
        leafletObject.addTo(map);
        addCallback(node);
      }
    });
  };

  /**
   * Removes a list of nodes from the map.
   * @param {Array} nodes - An array of nodes to be removed from the map.
   */
  function removeNodeListToHyperleaflet(nodes) {
    nodes.forEach((node) => {
      if (node.nodeType === 1 && node.matches('[data-id]')) {
        const [leafletObject] = deleteNodeFromHyperleaflet(node);
        leafletObject.remove();
        removeCallback(node);
      }
    });
  }

  return { addNoteListToHyperleaflet, removeNodeListToHyperleaflet };
}
